@@ -81,8 +81,11 @@ nasm_err_helpers(fatal_func, panic,    ERR_PANIC)
  * Strongly discourage warnings without level by require flags on warnings.
  * This means nasm_warn() is the equivalent of the -f variants of the
  * other ones.
+ *
+ * This is wrapped in a macro to be able to elide it if the warning is
+ * disabled, hence the extra underscore.
  */
-void nasm_warn(errflags flags, const char *fmt, ...)
+void nasm_warn_(errflags flags, const char *fmt, ...)
 {
 	nasm_do_error(ERR_WARNING, flags);
 }
@@ -129,9 +132,9 @@ void pop_warnings(void)
 	if (!ws->next) {
 		/*!
 		 *!warn-stack-empty [on] warning stack empty
-		 *!  a [WARNING POP] directive was executed when
+		 *!  a \c{[WARNING POP]} directive was executed when
 		 *!  the warning stack is empty. This is treated
-		 *!  as a [WARNING *all] directive.
+		 *!  as a \c{[WARNING *all]} directive.
 		 */
 		nasm_warn(WARN_WARN_STACK_EMPTY, "warning stack empty");
 	} else {
@@ -277,7 +280,7 @@ bool set_warning_status(const char *value)
 
         if (!ok && value) {
             /*!
-             *!unknown-warning [off] unknown warning in -W/-w or warning directive
+             *!unknown-warning [off] unknown warning in \c{-W}/\c{-w} or warning directive
              *!  warns about a \c{-w} or \c{-W} option or a \c{[WARNING]} directive
              *!  that contains an unknown warning name or is otherwise not possible to process.
              */
