@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 2007-2020 The NASM Authors - All Rights Reserved
+ *   Copyright 2007-2023 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -54,6 +54,8 @@
 /* On Microsoft platforms we support multibyte character sets in filenames */
 #define _MBCS 1
 
+#include "autoconf/attribute.h"
+
 #ifdef HAVE_CONFIG_H
 # include "config/config.h"
 #else
@@ -252,6 +254,13 @@ static inline void *mempcpy(void *dst, const void *src, size_t n)
 }
 #endif
 
+#ifndef HAVE_MEMPSET
+static inline void *mempset(void *dst, int c, size_t n)
+{
+    return (char *)memset(dst, c, n) + n;
+}
+#endif
+
 /*
  * Hack to support external-linkage inline functions
  */
