@@ -1,7 +1,7 @@
 ;;; nasmtok.el --- lists of NASM assembler tokens
 ;;;
 ;;; This file contains list of tokens from the NASM x86
-;;; assembler, automatically extracted from NASM 2.16.01.
+;;; assembler, automatically extracted from NASM 2.16.03.
 ;;;
 ;;; This file is intended to be (require)d from a `nasm-mode'
 ;;; major mode definition.
@@ -9,7 +9,7 @@
 (defconst nasm-decorator
   '("1to16" "1to2" "1to32" "1to4" "1to8" "rd-sae" "rn-sae" "ru-sae" "rz-sae"
     "sae" "z")
-  "NASM 2.16.01 decorator tokens for `nasm-mode'.")
+  "NASM 2.16.03 decorator tokens for `nasm-mode'.")
 
 (defconst nasm-directive
   '("absolute" "bits" "common" "cpu" "debug" "default" "export" "extern"
@@ -18,7 +18,7 @@
     "nodepend" "noseclabels" "options" "org" "osabi" "pragma" "prefix"
     "required" "safeseh" "sectalign" "section" "segment" "static"
     "subsections_via_symbols" "suffix" "uppercase" "warning")
-  "NASM 2.16.01 directive tokens for `nasm-mode'.")
+  "NASM 2.16.03 directive tokens for `nasm-mode'.")
 
 (defconst nasm-function
   '("__?bfloat16?__" "__?float128h?__" "__?float128l?__" "__?float16?__"
@@ -30,7 +30,7 @@
     "float16" "float32" "float64" "float8" "float80e" "float80m" "ilog2c"
     "ilog2e" "ilog2f" "ilog2w" "infinity" "nan" "qnan" "snan" "utf16"
     "utf16be" "utf16le" "utf32" "utf32be" "utf32le")
-  "NASM 2.16.01 function tokens for `nasm-mode'.")
+  "NASM 2.16.03 function tokens for `nasm-mode'.")
 
 (defconst nasm-instruction
   '("aaa" "aad" "aadd" "aam" "aand" "aas" "adc" "adcx" "add" "addpd" "addps"
@@ -65,53 +65,54 @@
     "cvttps2pi" "cvttsd2si" "cvttss2si" "cwd" "cwde" "daa" "das" "db" "dd"
     "dec" "div" "divpd" "divps" "divsd" "divss" "dmint" "do" "dppd" "dpps"
     "dq" "dt" "dw" "dy" "dz" "emms" "encls" "enclu" "enclv" "endbr32"
-    "endbr64" "enqcmd" "enqcmds" "enter" "equ" "extractps" "extrq" "f2xm1"
-    "fabs" "fadd" "faddp" "fbld" "fbstp" "fchs" "fclex" "fcmovb" "fcmovbe"
-    "fcmove" "fcmovnb" "fcmovnbe" "fcmovne" "fcmovnu" "fcmovu" "fcom" "fcomi"
-    "fcomip" "fcomp" "fcompp" "fcos" "fdecstp" "fdisi" "fdiv" "fdivp" "fdivr"
-    "fdivrp" "femms" "feni" "ffree" "ffreep" "fiadd" "ficom" "ficomp" "fidiv"
-    "fidivr" "fild" "fimul" "fincstp" "finit" "fist" "fistp" "fisttp" "fisub"
-    "fisubr" "fld" "fld1" "fldcw" "fldenv" "fldl2e" "fldl2t" "fldlg2"
-    "fldln2" "fldpi" "fldz" "fmul" "fmulp" "fnclex" "fndisi" "fneni" "fninit"
-    "fnop" "fnsave" "fnstcw" "fnstenv" "fnstsw" "fpatan" "fprem" "fprem1"
-    "fptan" "frndint" "frstor" "fsave" "fscale" "fsetpm" "fsin" "fsincos"
-    "fsqrt" "fst" "fstcw" "fstenv" "fstp" "fstsw" "fsub" "fsubp" "fsubr"
-    "fsubrp" "ftst" "fucom" "fucomi" "fucomip" "fucomp" "fucompp" "fwait"
-    "fxam" "fxch" "fxrstor" "fxrstor64" "fxsave" "fxsave64" "fxtract" "fyl2x"
-    "fyl2xp1" "getsec" "gf2p8affineinvqb" "gf2p8affineqb" "gf2p8mulb"
-    "haddpd" "haddps" "hint_nop0" "hint_nop1" "hint_nop10" "hint_nop11"
-    "hint_nop12" "hint_nop13" "hint_nop14" "hint_nop15" "hint_nop16"
-    "hint_nop17" "hint_nop18" "hint_nop19" "hint_nop2" "hint_nop20"
-    "hint_nop21" "hint_nop22" "hint_nop23" "hint_nop24" "hint_nop25"
-    "hint_nop26" "hint_nop27" "hint_nop28" "hint_nop29" "hint_nop3"
-    "hint_nop30" "hint_nop31" "hint_nop32" "hint_nop33" "hint_nop34"
-    "hint_nop35" "hint_nop36" "hint_nop37" "hint_nop38" "hint_nop39"
-    "hint_nop4" "hint_nop40" "hint_nop41" "hint_nop42" "hint_nop43"
-    "hint_nop44" "hint_nop45" "hint_nop46" "hint_nop47" "hint_nop48"
-    "hint_nop49" "hint_nop5" "hint_nop50" "hint_nop51" "hint_nop52"
-    "hint_nop53" "hint_nop54" "hint_nop55" "hint_nop56" "hint_nop57"
-    "hint_nop58" "hint_nop59" "hint_nop6" "hint_nop60" "hint_nop61"
-    "hint_nop62" "hint_nop63" "hint_nop7" "hint_nop8" "hint_nop9" "hlt"
-    "hreset" "hsubpd" "hsubps" "ibts" "icebp" "idiv" "imul" "in" "inc"
-    "incbin" "incsspd" "incsspq" "insb" "insd" "insertps" "insertq" "insw"
-    "int" "int01" "int03" "int1" "int3" "into" "invd" "invept" "invlpg"
-    "invlpga" "invpcid" "invvpid" "iret" "iretd" "iretq" "iretw" "ja" "jae"
-    "jb" "jbe" "jc" "jcxz" "je" "jecxz" "jg" "jge" "jl" "jle" "jmp" "jmpe"
-    "jna" "jnae" "jnb" "jnbe" "jnc" "jne" "jng" "jnge" "jnl" "jnle" "jno"
-    "jnp" "jns" "jnz" "jo" "jp" "jpe" "jpo" "jrcxz" "js" "jz" "kadd" "kaddb"
-    "kaddd" "kaddq" "kaddw" "kand" "kandb" "kandd" "kandn" "kandnb" "kandnd"
-    "kandnq" "kandnw" "kandq" "kandw" "kmov" "kmovb" "kmovd" "kmovq" "kmovw"
-    "knot" "knotb" "knotd" "knotq" "knotw" "kor" "korb" "kord" "korq"
-    "kortest" "kortestb" "kortestd" "kortestq" "kortestw" "korw" "kshiftl"
-    "kshiftlb" "kshiftld" "kshiftlq" "kshiftlw" "kshiftr" "kshiftrb"
-    "kshiftrd" "kshiftrq" "kshiftrw" "ktest" "ktestb" "ktestd" "ktestq"
-    "ktestw" "kunpck" "kunpckbw" "kunpckdq" "kunpckwd" "kxnor" "kxnorb"
-    "kxnord" "kxnorq" "kxnorw" "kxor" "kxorb" "kxord" "kxorq" "kxorw" "lahf"
-    "lar" "lddqu" "ldmxcsr" "lds" "ldtilecfg" "lea" "leave" "les" "lfence"
-    "lfs" "lgdt" "lgs" "lidt" "lldt" "llwpcb" "lmsw" "loadall" "loadall286"
-    "lodsb" "lodsd" "lodsq" "lodsw" "loop" "loope" "loopne" "loopnz" "loopz"
-    "lsl" "lss" "ltr" "lwpins" "lwpval" "lzcnt" "maskmovdqu" "maskmovq"
-    "maxpd" "maxps" "maxsd" "maxss" "mfence" "minpd" "minps" "minsd" "minss"
+    "endbr64" "enqcmd" "enqcmds" "enter" "equ" "erets" "eretu" "extractps"
+    "extrq" "f2xm1" "fabs" "fadd" "faddp" "fbld" "fbstp" "fchs" "fclex"
+    "fcmovb" "fcmovbe" "fcmove" "fcmovnb" "fcmovnbe" "fcmovne" "fcmovnu"
+    "fcmovu" "fcom" "fcomi" "fcomip" "fcomp" "fcompp" "fcos" "fdecstp"
+    "fdisi" "fdiv" "fdivp" "fdivr" "fdivrp" "femms" "feni" "ffree" "ffreep"
+    "fiadd" "ficom" "ficomp" "fidiv" "fidivr" "fild" "fimul" "fincstp"
+    "finit" "fist" "fistp" "fisttp" "fisub" "fisubr" "fld" "fld1" "fldcw"
+    "fldenv" "fldl2e" "fldl2t" "fldlg2" "fldln2" "fldpi" "fldz" "fmul"
+    "fmulp" "fnclex" "fndisi" "fneni" "fninit" "fnop" "fnsave" "fnstcw"
+    "fnstenv" "fnstsw" "fpatan" "fprem" "fprem1" "fptan" "frndint" "frstor"
+    "fsave" "fscale" "fsetpm" "fsin" "fsincos" "fsqrt" "fst" "fstcw" "fstenv"
+    "fstp" "fstsw" "fsub" "fsubp" "fsubr" "fsubrp" "ftst" "fucom" "fucomi"
+    "fucomip" "fucomp" "fucompp" "fwait" "fxam" "fxch" "fxrstor" "fxrstor64"
+    "fxsave" "fxsave64" "fxtract" "fyl2x" "fyl2xp1" "getsec"
+    "gf2p8affineinvqb" "gf2p8affineqb" "gf2p8mulb" "haddpd" "haddps"
+    "hint_nop0" "hint_nop1" "hint_nop10" "hint_nop11" "hint_nop12"
+    "hint_nop13" "hint_nop14" "hint_nop15" "hint_nop16" "hint_nop17"
+    "hint_nop18" "hint_nop19" "hint_nop2" "hint_nop20" "hint_nop21"
+    "hint_nop22" "hint_nop23" "hint_nop24" "hint_nop25" "hint_nop26"
+    "hint_nop27" "hint_nop28" "hint_nop29" "hint_nop3" "hint_nop30"
+    "hint_nop31" "hint_nop32" "hint_nop33" "hint_nop34" "hint_nop35"
+    "hint_nop36" "hint_nop37" "hint_nop38" "hint_nop39" "hint_nop4"
+    "hint_nop40" "hint_nop41" "hint_nop42" "hint_nop43" "hint_nop44"
+    "hint_nop45" "hint_nop46" "hint_nop47" "hint_nop48" "hint_nop49"
+    "hint_nop5" "hint_nop50" "hint_nop51" "hint_nop52" "hint_nop53"
+    "hint_nop54" "hint_nop55" "hint_nop56" "hint_nop57" "hint_nop58"
+    "hint_nop59" "hint_nop6" "hint_nop60" "hint_nop61" "hint_nop62"
+    "hint_nop63" "hint_nop7" "hint_nop8" "hint_nop9" "hlt" "hreset" "hsubpd"
+    "hsubps" "ibts" "icebp" "idiv" "imul" "in" "inc" "incbin" "incsspd"
+    "incsspq" "insb" "insd" "insertps" "insertq" "insw" "int" "int01" "int03"
+    "int1" "int3" "into" "invd" "invept" "invlpg" "invlpga" "invpcid"
+    "invvpid" "iret" "iretd" "iretq" "iretw" "ja" "jae" "jb" "jbe" "jc"
+    "jcxz" "je" "jecxz" "jg" "jge" "jl" "jle" "jmp" "jmpe" "jna" "jnae" "jnb"
+    "jnbe" "jnc" "jne" "jng" "jnge" "jnl" "jnle" "jno" "jnp" "jns" "jnz" "jo"
+    "jp" "jpe" "jpo" "jrcxz" "js" "jz" "kadd" "kaddb" "kaddd" "kaddq" "kaddw"
+    "kand" "kandb" "kandd" "kandn" "kandnb" "kandnd" "kandnq" "kandnw"
+    "kandq" "kandw" "kmov" "kmovb" "kmovd" "kmovq" "kmovw" "knot" "knotb"
+    "knotd" "knotq" "knotw" "kor" "korb" "kord" "korq" "kortest" "kortestb"
+    "kortestd" "kortestq" "kortestw" "korw" "kshiftl" "kshiftlb" "kshiftld"
+    "kshiftlq" "kshiftlw" "kshiftr" "kshiftrb" "kshiftrd" "kshiftrq"
+    "kshiftrw" "ktest" "ktestb" "ktestd" "ktestq" "ktestw" "kunpck"
+    "kunpckbw" "kunpckdq" "kunpckwd" "kxnor" "kxnorb" "kxnord" "kxnorq"
+    "kxnorw" "kxor" "kxorb" "kxord" "kxorq" "kxorw" "lahf" "lar" "lddqu"
+    "ldmxcsr" "lds" "ldtilecfg" "lea" "leave" "les" "lfence" "lfs" "lgdt"
+    "lgs" "lidt" "lkgs" "lldt" "llwpcb" "lmsw" "loadall" "loadall286" "lodsb"
+    "lodsd" "lodsq" "lodsw" "loop" "loope" "loopne" "loopnz" "loopz" "lsl"
+    "lss" "ltr" "lwpins" "lwpval" "lzcnt" "maskmovdqu" "maskmovq" "maxpd"
+    "maxps" "maxsd" "maxss" "mfence" "minpd" "minps" "minsd" "minss"
     "monitor" "monitorx" "montmul" "mov" "movapd" "movaps" "movbe" "movd"
     "movddup" "movdir64b" "movdiri" "movdq2q" "movdqa" "movdqu" "movhlps"
     "movhpd" "movhps" "movlhps" "movlpd" "movlps" "movmskpd" "movmskps"
@@ -377,24 +378,26 @@
     "vscalefss" "vscatterdpd" "vscatterdps" "vscatterpf0dpd" "vscatterpf0dps"
     "vscatterpf0qpd" "vscatterpf0qps" "vscatterpf1dpd" "vscatterpf1dps"
     "vscatterpf1qpd" "vscatterpf1qps" "vscatterqpd" "vscatterqps"
-    "vshuff32x4" "vshuff64x2" "vshufi32x4" "vshufi64x2" "vshufpd" "vshufps"
-    "vsqrtpd" "vsqrtph" "vsqrtps" "vsqrtsd" "vsqrtsh" "vsqrtss" "vstmxcsr"
-    "vsubpd" "vsubph" "vsubps" "vsubsd" "vsubsh" "vsubss" "vtestpd" "vtestps"
-    "vucomisd" "vucomish" "vucomiss" "vunpckhpd" "vunpckhps" "vunpcklpd"
-    "vunpcklps" "vxorpd" "vxorps" "vzeroall" "vzeroupper" "wbinvd" "wbnoinvd"
-    "wrfsbase" "wrgsbase" "wrmsr" "wrmsrlist" "wrmsrns" "wrpkru" "wrshr"
-    "wrssd" "wrssq" "wrussd" "wrussq" "xabort" "xadd" "xbegin" "xbts" "xchg"
-    "xcryptcbc" "xcryptcfb" "xcryptctr" "xcryptecb" "xcryptofb" "xend"
-    "xgetbv" "xlat" "xlatb" "xor" "xorpd" "xorps" "xresldtrk" "xrstor"
-    "xrstor64" "xrstors" "xrstors64" "xsave" "xsave64" "xsavec" "xsavec64"
-    "xsaveopt" "xsaveopt64" "xsaves" "xsaves64" "xsetbv" "xsha1" "xsha256"
-    "xstore" "xsusldtrk" "xtest")
-  "NASM 2.16.01 instruction tokens for `nasm-mode'.")
+    "vsha512msg1" "vsha512msg2" "vsha512rnds2" "vshuff32x4" "vshuff64x2"
+    "vshufi32x4" "vshufi64x2" "vshufpd" "vshufps" "vsm3msg1" "vsm3msg2"
+    "vsm3rnds2" "vsm4key4" "vsm4rnds4" "vsqrtpd" "vsqrtph" "vsqrtps"
+    "vsqrtsd" "vsqrtsh" "vsqrtss" "vstmxcsr" "vsubpd" "vsubph" "vsubps"
+    "vsubsd" "vsubsh" "vsubss" "vtestpd" "vtestps" "vucomisd" "vucomish"
+    "vucomiss" "vunpckhpd" "vunpckhps" "vunpcklpd" "vunpcklps" "vxorpd"
+    "vxorps" "vzeroall" "vzeroupper" "wbinvd" "wbnoinvd" "wrfsbase"
+    "wrgsbase" "wrmsr" "wrmsrlist" "wrmsrns" "wrpkru" "wrshr" "wrssd" "wrssq"
+    "wrussd" "wrussq" "xabort" "xadd" "xbegin" "xbts" "xchg" "xcryptcbc"
+    "xcryptcfb" "xcryptctr" "xcryptecb" "xcryptofb" "xend" "xgetbv" "xlat"
+    "xlatb" "xor" "xorpd" "xorps" "xresldtrk" "xrstor" "xrstor64" "xrstors"
+    "xrstors64" "xsave" "xsave64" "xsavec" "xsavec64" "xsaveopt" "xsaveopt64"
+    "xsaves" "xsaves64" "xsetbv" "xsha1" "xsha256" "xstore" "xsusldtrk"
+    "xtest")
+  "NASM 2.16.03 instruction tokens for `nasm-mode'.")
 
 (defconst nasm-operator
   '("!=" "%%" "&&" "//" "<<" "<<<" "<=" "<=>" "<>" "==" ">=" ">>" ">>>" "?"
     "^^" "||")
-  "NASM 2.16.01 operator tokens for `nasm-mode'.")
+  "NASM 2.16.03 operator tokens for `nasm-mode'.")
 
 (defconst nasm-pp-directive
   '("%aliases" "%arg" "%assign" "%clear" "%defalias" "%define" "%defstr"
@@ -416,13 +419,13 @@
     "%null" "%pathsearch" "%pop" "%pragma" "%push" "%rep" "%repl" "%require"
     "%rmacro" "%rotate" "%stacksize" "%strcat" "%strlen" "%substr" "%undef"
     "%undefalias" "%unimacro" "%unmacro" "%use" "%warning" "%xdefine")
-  "NASM 2.16.01 pp-directive tokens for `nasm-mode'.")
+  "NASM 2.16.03 pp-directive tokens for `nasm-mode'.")
 
 (defconst nasm-prefix
   '("a16" "a32" "a64" "asp" "bnd" "evex" "lock" "nobnd" "o16" "o32" "o64"
     "osp" "rep" "repe" "repne" "repnz" "repz" "rex" "times" "vex" "vex2"
     "vex3" "wait" "xacquire" "xrelease")
-  "NASM 2.16.01 prefix tokens for `nasm-mode'.")
+  "NASM 2.16.03 prefix tokens for `nasm-mode'.")
 
 (defconst nasm-register
   '("ah" "al" "ax" "bh" "bl" "bnd0" "bnd1" "bnd2" "bnd3" "bp" "bpl" "bx" "ch"
@@ -449,13 +452,13 @@
     "zmm19" "zmm2" "zmm20" "zmm21" "zmm22" "zmm23" "zmm24" "zmm25" "zmm26"
     "zmm27" "zmm28" "zmm29" "zmm3" "zmm30" "zmm31" "zmm4" "zmm5" "zmm6"
     "zmm7" "zmm8" "zmm9")
-  "NASM 2.16.01 register tokens for `nasm-mode'.")
+  "NASM 2.16.03 register tokens for `nasm-mode'.")
 
 (defconst nasm-size
   '("byte" "dword" "oword" "qword" "tword" "word" "yword" "zword")
-  "NASM 2.16.01 size tokens for `nasm-mode'.")
+  "NASM 2.16.03 size tokens for `nasm-mode'.")
 
 (defconst nasm-special
   '("abs" "dup" "far" "long" "near" "nosplit" "ptr" "rel" "seg" "short"
     "strict" "to" "wrt")
-  "NASM 2.16.01 special tokens for `nasm-mode'.")
+  "NASM 2.16.03 special tokens for `nasm-mode'.")
