@@ -38,11 +38,8 @@
 
 #include "compiler.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <limits.h>
+#include <ctype.h>              /* For toupper() */
+#include "nctype.h"
 
 #include "nasm.h"
 #include "nasmlib.h"
@@ -67,7 +64,7 @@
  */
 
 /*
- * These routines are built around the ObjRecord data struture.  An ObjRecord
+ * These routines are built around the ObjRecord data structure.  An ObjRecord
  * holds an object file record that may be under construction or complete.
  *
  * A major function of these routines is to support continuation of an obj
@@ -202,7 +199,7 @@ static ObjRecord *obj_clear(ObjRecord * orp)
 
 /*
  * Emit an ObjRecord structure.  (Never reallocates).
- * The record is written out preceeded (recursively) by its previous part (if
+ * The record is written out proceeded (recursively) by its previous part (if
  * any) and followed (recursively) by its child (if any).
  * The previous part and the child are freed.  The main ObjRecord is cleared,
  * not freed.
@@ -249,7 +246,7 @@ static ObjRecord *obj_new(void)
 /*
  * Advance to the next record because the existing one is full or its x_size
  * is incompatible.
- * Any uncommited data is moved into the next record.
+ * Any uncommitted data is moved into the next record.
  */
 static ObjRecord *obj_bump(ObjRecord * orp)
 {
@@ -297,7 +294,7 @@ static ObjRecord *obj_check(ObjRecord * orp, int size)
 }
 
 /*
- * All data written so far is commited to the current record (won't be moved to
+ * All data written so far is committed to the current record (won't be moved to
  * the next record in case of continuation).
  */
 static ObjRecord *obj_commit(ObjRecord * orp)
@@ -427,6 +424,12 @@ static ObjRecord *obj_name(ObjRecord * orp, const char *name)
     int len = strlen(name);
     uint8_t *ptr;
 
+    if (len > UINT8_MAX) {
+        nasm_warn(WARN_OTHER, "truncating object name `%.64s...' to %u bytes",
+                  name, UINT8_MAX);
+        len = UINT8_MAX;
+    }
+
     orp = obj_check(orp, len + 1);
     ptr = orp->buf + orp->used;
     *ptr++ = len;
@@ -580,6 +583,7 @@ static struct Segment {
     struct Group *grp;          /* the group it beint32_ts to */
     uint32_t currentpos;
     int32_t align;                 /* can be SEG_ABS + absolute addr */
+    int64_t pass_last_seen;
     struct Public *pubhead, **pubtail, *lochead, **loctail;
     char *segclass, *overlay;   /* `class' is a C++ keyword :-) */
     ObjRecord *orp;
@@ -634,9 +638,9 @@ static const struct dfmt borland_debug_form;
 /* The current segment */
 static struct Segment *current_seg;
 
-static int32_t obj_segment(char *, int, int *);
+static int32_t obj_segment(char *, int *);
 static void obj_write_file(void);
-static enum directive_result obj_directive(enum directive, char *, int);
+static enum directive_result obj_directive(enum directive, char *);
 
 static void obj_init(void)
 {
@@ -773,11 +777,9 @@ static void obj_deflabel(char *name, int32_t segment,
     int i;
     bool used_special = false;   /* have we used the special text? */
 
-#if defined(DEBUG) && DEBUG>2
-    nasm_error(ERR_DEBUG,
-            " obj_deflabel: %s, seg=%"PRIx32", off=%"PRIx64", is_global=%d, %s\n",
-            name, segment, offset, is_global, special);
-#endif
+    if (debug_level(2))
+        nasm_debug(" obj_deflabel: %s, seg=%"PRIx32", off=%"PRIx64", is_global=%d, %s\n",
+                   name, segment, offset, is_global, special);
 
     /*
      * If it's a special-retry from pass two, discard it.
@@ -795,7 +797,7 @@ static void obj_deflabel(char *name, int32_t segment,
             obj_entry_ofs = offset;
             return;
         }
-        nasm_error(ERR_NONFATAL, "unrecognised special symbol `%s'", name);
+        nasm_nonfatal("unrecognised special symbol `%s'", name);
     }
 
     /*
@@ -826,8 +828,8 @@ static void obj_deflabel(char *name, int32_t segment,
             pub->segment = (segment == NO_SEG ? 0 : segment & ~SEG_ABS);
         }
         if (special)
-            nasm_error(ERR_NONFATAL, "OBJ supports no special symbol features"
-                  " for this symbol type");
+            nasm_nonfatal("OBJ supports no special symbol features"
+                          " for this symbol type");
         return;
     }
 
@@ -837,9 +839,9 @@ static void obj_deflabel(char *name, int32_t segment,
      * `first_seg'.
      */
     if (!any_segs && segment == first_seg) {
-        int tempint;            /* ignored */
-        if (segment != obj_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_panic(0, "strange segment conditions in OBJ driver");
+        int tempint = 0;
+        if (segment != obj_segment("__NASMDEFSEG", &tempint))
+            nasm_panic("strange segment conditions in OBJ driver");
     }
 
     for (seg = seghead; seg && is_global; seg = seg->next)
@@ -855,9 +857,8 @@ static void obj_deflabel(char *name, int32_t segment,
             loc->offset = offset;
 
             if (special)
-                nasm_error(ERR_NONFATAL,
-                      "OBJ supports no special symbol features"
-                      " for this symbol type");
+                nasm_nonfatal("OBJ supports no special symbol features"
+                              " for this symbol type");
             return;
         }
 
@@ -922,8 +923,8 @@ static void obj_deflabel(char *name, int32_t segment,
             obj_ext_set_defwrt(ext, p);
             special += len;
             if (*special && *special != ':')
-                nasm_error(ERR_NONFATAL, "`:' expected in special symbol"
-                      " text for `%s'", ext->name);
+                nasm_nonfatal("`:' expected in special symbol"
+                              " text for `%s'", ext->name);
             else if (*special == ':')
                 special++;
         }
@@ -936,18 +937,16 @@ static void obj_deflabel(char *name, int32_t segment,
             if (ext->commonsize)
                 ext->commonelem = 1;
             else
-                nasm_error(ERR_NONFATAL,
-                      "`%s': `far' keyword may only be applied"
-                      " to common variables\n", ext->name);
+                nasm_nonfatal("`%s': `far' keyword may only be applied"
+                              " to common variables\n", ext->name);
             special += 3;
             special += strspn(special, " \t");
         } else if (!nasm_strnicmp(special, "near", 4)) {
             if (ext->commonsize)
                 ext->commonelem = 0;
             else
-                nasm_error(ERR_NONFATAL,
-                      "`%s': `far' keyword may only be applied"
-                      " to common variables\n", ext->name);
+                nasm_nonfatal("`%s': `far' keyword may only be applied"
+                              " to common variables\n", ext->name);
             special += 4;
             special += strspn(special, " \t");
         }
@@ -971,16 +970,15 @@ static void obj_deflabel(char *name, int32_t segment,
                 e = evaluate(stdscan, NULL, &tokval, NULL, 1, NULL);
                 if (e) {
                     if (!is_simple(e))
-                        nasm_error(ERR_NONFATAL, "cannot use relocatable"
-                              " expression as common-variable element size");
+                        nasm_nonfatal("cannot use relocatable"
+                                      " expression as common-variable element size");
                     else
                         ext->commonelem = reloc_value(e);
                 }
                 special = stdscan_get();
             } else {
-                nasm_error(ERR_NONFATAL,
-                      "`%s': element-size specifications only"
-                      " apply to common variables", ext->name);
+                nasm_nonfatal("`%s': element-size specifications only"
+                              " apply to common variables", ext->name);
                 while (*special && *special != ':')
                     special++;
                 if (*special == ':')
@@ -1010,8 +1008,8 @@ static void obj_deflabel(char *name, int32_t segment,
     ext->index = ++externals;
 
     if (special && !used_special)
-        nasm_error(ERR_NONFATAL, "OBJ supports no special symbol features"
-              " for this symbol type");
+        nasm_nonfatal("OBJ supports no special symbol features"
+                      " for this symbol type");
 }
 
 /* forward declaration */
@@ -1028,34 +1026,24 @@ static void obj_out(int32_t segto, const void *data,
     struct Segment *seg;
     ObjRecord *orp;
 
-    /*
-     * handle absolute-assembly (structure definitions)
-     */
-    if (segto == NO_SEG) {
-        if (type != OUT_RESERVE)
-            nasm_error(ERR_NONFATAL, "attempt to assemble code in [ABSOLUTE]"
-                  " space");
-        return;
-    }
-
     /*
      * If `any_segs' is still false, we must define a default
      * segment.
      */
     if (!any_segs) {
-        int tempint;            /* ignored */
-        if (segto != obj_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_panic(0, "strange segment conditions in OBJ driver");
+        int tempint = 0;
+        if (segto != obj_segment("__NASMDEFSEG", &tempint))
+            nasm_panic("strange segment conditions in OBJ driver");
     }
 
     /*
-     * Find the segment we are targetting.
+     * Find the segment we are targeting.
      */
     for (seg = seghead; seg; seg = seg->next)
         if (seg->index == segto)
             break;
     if (!seg)
-        nasm_panic(0, "code directed to nonexistent segment?");
+        nasm_panic("code directed to nonexistent segment?");
 
     orp = seg->orp;
     orp->parm[0] = seg->currentpos;
@@ -1089,11 +1077,11 @@ static void obj_out(int32_t segto, const void *data,
             size = abs((int)size);
 
         if (segment == NO_SEG && type != OUT_ADDRESS)
-            nasm_error(ERR_NONFATAL, "relative call to absolute address not"
-                  " supported by OBJ format");
+            nasm_nonfatal("relative call to absolute address not"
+                          " supported by OBJ format");
         if (segment >= SEG_ABS)
-            nasm_error(ERR_NONFATAL, "far-absolute relocations not supported"
-                  " by OBJ format");
+            nasm_nonfatal("far-absolute relocations not supported"
+                          " by OBJ format");
 
         ldata = *(int64_t *)data;
         if (type != OUT_ADDRESS) {
@@ -1138,8 +1126,8 @@ static void obj_out(int32_t segto, const void *data,
 
 	switch (size) {
 	default:
-	    nasm_error(ERR_NONFATAL, "OBJ format can only handle 16- or "
-		       "32-byte relocations");
+            nasm_nonfatal("OBJ format can only handle 16- or "
+                          "32-byte relocations");
 	    segment = NO_SEG;	/* Don't actually generate a relocation */
 	    break;
 	case 2:
@@ -1162,8 +1150,8 @@ static void obj_out(int32_t segto, const void *data,
              */
             rsize = 2;
             if (ldata & 0xFFFF)
-                nasm_error(ERR_NONFATAL, "OBJ format cannot handle complex"
-                      " dword-size segment base references");
+                nasm_nonfatal("OBJ format cannot handle complex"
+                              " dword-size segment base references");
         }
         if (segment != NO_SEG)
             obj_write_fixup(orp, rsize,
@@ -1174,8 +1162,7 @@ static void obj_out(int32_t segto, const void *data,
     }
 
     default:
-	nasm_error(ERR_NONFATAL,
-		   "Relocation type not supported by output format");
+        nasm_nonfatal("Relocation type not supported by output format");
 	/* fall through */
 
     case OUT_RESERVE:
@@ -1201,8 +1188,8 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
     ObjRecord *forp;
 
     if (bytes != 2 && bytes != 4) {
-        nasm_error(ERR_NONFATAL, "`obj' output driver does not support"
-		   " %d-bit relocations", bytes << 3);
+        nasm_nonfatal("`obj' output driver does not support"
+                      " %d-bit relocations", bytes << 3);
         return;
     }
 
@@ -1223,7 +1210,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
         locat = FIX_16_SELECTOR;
         seg--;
         if (bytes != 2)
-            nasm_panic(0, "OBJ: 4-byte segment base fixup got"
+            nasm_panic("OBJ: 4-byte segment base fixup got"
                   " through sanity check");
     } else {
         base = false;
@@ -1269,8 +1256,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
             if (eb)
                 method = 6, e = eb->exts[i], tidx = e->index;
             else
-                nasm_panic(0,
-                      "unrecognised segment value in obj_write_fixup");
+                nasm_panic("unrecognised segment value in obj_write_fixup");
         }
     }
 
@@ -1293,8 +1279,8 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
             else if (e->defwrt_type == DEFWRT_GROUP)
                 method |= 0x10, fidx = e->defwrt_ptr.grp->obj_index;
             else {
-                nasm_error(ERR_NONFATAL, "default WRT specification for"
-                      " external `%s' unresolved", e->name);
+                nasm_nonfatal("default WRT specification for"
+                              " external `%s' unresolved", e->name);
                 method |= 0x50, fidx = -1;      /* got to do _something_ */
             }
         } else
@@ -1328,8 +1314,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
                 if (eb)
                     method |= 0x20, fidx = eb->exts[i]->index;
                 else
-                    nasm_panic(0,
-                          "unrecognised WRT value in obj_write_fixup");
+                    nasm_panic("unrecognised WRT value in obj_write_fixup");
             }
         }
     }
@@ -1341,7 +1326,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
     obj_commit(forp);
 }
 
-static int32_t obj_segment(char *name, int pass, int *bits)
+static int32_t obj_segment(char *name, int *bits)
 {
     /*
      * We call the label manager here to define a name for the new
@@ -1350,10 +1335,9 @@ static int32_t obj_segment(char *name, int pass, int *bits)
      * using the pointer it gets passed. That way we save memory,
      * by sponging off the label manager.
      */
-#if defined(DEBUG) && DEBUG>=3
-    nasm_error(ERR_DEBUG, " obj_segment: < %s >, pass=%d, *bits=%d\n",
-            name, pass, *bits);
-#endif
+    if (debug_level(3))
+        nasm_debug(" obj_segment: < %s >, *bits=%d\n", name, *bits);
+
     if (!name) {
         *bits = 16;
         current_seg = NULL;
@@ -1397,14 +1381,15 @@ static int32_t obj_segment(char *name, int pass, int *bits)
                 break;
 
             if (!strcmp(seg->name, name)) {
-                if (attrs > 0 && pass == 1)
-                    nasm_error(ERR_WARNING, "segment attributes specified on"
-                          " redeclaration of segment: ignoring");
+                if (attrs > 0 && seg->pass_last_seen == pass_count())
+                    nasm_warn(WARN_OTHER, "segment attributes specified on"
+                              " redeclaration of segment: ignoring");
                 if (seg->use32)
                     *bits = 32;
                 else
                     *bits = 16;
                 current_seg = seg;
+                seg->pass_last_seen = pass_count();
                 return seg->index;
             }
         }
@@ -1473,12 +1458,12 @@ static int32_t obj_segment(char *name, int pass, int *bits)
                     if (!strcmp(grp->name, "FLAT"))
                         break;
                 if (!grp) {
-                    obj_directive(D_GROUP, "FLAT", 1);
+                    obj_directive(D_GROUP, "FLAT");
                     for (grp = grphead; grp; grp = grp->next)
                         if (!strcmp(grp->name, "FLAT"))
                             break;
                     if (!grp)
-                        nasm_panic(0, "failure to define FLAT?!");
+                        nasm_panic("failure to define FLAT?!");
                 }
                 seg->grp = grp;
             } else if (!nasm_strnicmp(p, "class=", 6))
@@ -1489,8 +1474,7 @@ static int32_t obj_segment(char *name, int pass, int *bits)
                 seg->align = readnum(p + 6, &rn_error);
                 if (rn_error) {
                     seg->align = 1;
-                    nasm_error(ERR_NONFATAL, "segment alignment should be"
-                          " numeric");
+                    nasm_nonfatal("segment alignment should be numeric");
                 }
                 switch (seg->align) {
                 case 1:        /* BYTE */
@@ -1501,38 +1485,35 @@ static int32_t obj_segment(char *name, int pass, int *bits)
                 case 4096:     /* PharLap extension */
                     break;
                 case 8:
-                    nasm_error(ERR_WARNING,
-                          "OBJ format does not support alignment"
-                          " of 8: rounding up to 16");
+                    nasm_warn(WARN_OTHER, "OBJ format does not support alignment"
+                              " of 8: rounding up to 16");
                     seg->align = 16;
                     break;
                 case 32:
                 case 64:
                 case 128:
-                    nasm_error(ERR_WARNING,
-                          "OBJ format does not support alignment"
-                          " of %d: rounding up to 256", seg->align);
+                    nasm_warn(WARN_OTHER, "OBJ format does not support alignment"
+                              " of %d: rounding up to 256", seg->align);
                     seg->align = 256;
                     break;
                 case 512:
                 case 1024:
                 case 2048:
-                    nasm_error(ERR_WARNING,
-                          "OBJ format does not support alignment"
-                          " of %d: rounding up to 4096", seg->align);
+                    nasm_warn(WARN_OTHER, "OBJ format does not support alignment"
+                              " of %d: rounding up to 4096", seg->align);
                     seg->align = 4096;
                     break;
                 default:
-                    nasm_error(ERR_NONFATAL, "invalid alignment value %d",
-                          seg->align);
+                    nasm_nonfatal("invalid alignment value %d",
+                                  seg->align);
                     seg->align = 1;
                     break;
                 }
             } else if (!nasm_strnicmp(p, "absolute=", 9)) {
                 seg->align = SEG_ABS + readnum(p + 9, &rn_error);
                 if (rn_error)
-                    nasm_error(ERR_NONFATAL, "argument to `absolute' segment"
-                          " attribute should be numeric");
+                    nasm_nonfatal("argument to `absolute' segment"
+                                  " attribute should be numeric");
             }
         }
 
@@ -1556,10 +1537,9 @@ static int32_t obj_segment(char *name, int pass, int *bits)
                     grp->segs[i] = grp->segs[grp->nindices];
                     grp->segs[grp->nindices++].index = seg->obj_index;
                     if (seg->grp)
-                        nasm_error(ERR_WARNING,
-				   "segment `%s' is already part of"
-				   " a group: first one takes precedence",
-				   seg->name);
+                        nasm_warn(WARN_OTHER, "segment `%s' is already part of"
+                                  " a group: first one takes precedence",
+                                  seg->name);
                     else
                         seg->grp = grp;
                 }
@@ -1593,13 +1573,13 @@ static int32_t obj_segment(char *name, int pass, int *bits)
 }
 
 static enum directive_result
-obj_directive(enum directive directive, char *value, int pass)
+obj_directive(enum directive directive, char *value)
 {
     switch (directive) {
     case D_GROUP:
     {
         char *p, *q, *v;
-        if (pass == 1) {
+        if (pass_first()) {     /* XXX */
             struct Group *grp;
             struct Segment *seg;
             struct External **extp;
@@ -1632,7 +1612,7 @@ obj_directive(enum directive directive, char *value, int pass)
             for (grp = grphead; grp; grp = grp->next) {
                 obj_idx++;
                 if (!strcmp(grp->name, v)) {
-                    nasm_error(ERR_NONFATAL, "group `%s' defined twice", v);
+                    nasm_nonfatal("group `%s' defined twice", v);
                     return DIRR_ERROR;
                 }
             }
@@ -1672,10 +1652,9 @@ obj_directive(enum directive directive, char *value, int pass)
                     grp->segs[grp->nentries++] = grp->segs[grp->nindices];
                     grp->segs[grp->nindices++].index = seg->obj_index;
                     if (seg->grp)
-                        nasm_error(ERR_WARNING,
-                              "segment `%s' is already part of"
-                              " a group: first one takes precedence",
-                              seg->name);
+                        nasm_warn(WARN_OTHER, "segment `%s' is already part of"
+                                  " a group: first one takes precedence",
+                                  seg->name);
                     else
                         seg->grp = grp;
                 } else {
@@ -1714,8 +1693,8 @@ obj_directive(enum directive directive, char *value, int pass)
     {
         char *q, *extname, *libname, *impname;
 
-        if (pass == 2)
-            return 1;           /* ignore in pass two */
+        if (!pass_first())      /* XXX */
+            return DIRR_OK;
         extname = q = value;
         while (*q && !nasm_isspace(*q))
             q++;
@@ -1737,8 +1716,8 @@ obj_directive(enum directive directive, char *value, int pass)
         impname = q;
 
         if (!*extname || !*libname)
-            nasm_error(ERR_NONFATAL, "`import' directive requires symbol name"
-                  " and library name");
+            nasm_nonfatal("`import' directive requires symbol name"
+                          " and library name");
         else {
             struct ImpDef *imp;
             bool err = false;
@@ -1764,7 +1743,7 @@ obj_directive(enum directive directive, char *value, int pass)
         int flags = 0;
         unsigned int ordinal = 0;
 
-        if (pass == 2)
+        if (!pass_first())
             return DIRR_OK;     /* ignore in pass two */
         intname = q = value;
         while (*q && !nasm_isspace(*q))
@@ -1785,7 +1764,7 @@ obj_directive(enum directive directive, char *value, int pass)
         }
 
         if (!*intname) {
-            nasm_error(ERR_NONFATAL, "`export' directive requires export name");
+            nasm_nonfatal("`export' directive requires export name");
             return DIRR_OK;
         }
         if (!*extname) {
@@ -1809,16 +1788,14 @@ obj_directive(enum directive directive, char *value, int pass)
                 bool err = false;
                 flags |= EXPDEF_MASK_PARMCNT & readnum(v + 5, &err);
                 if (err) {
-                    nasm_error(ERR_NONFATAL,
-                          "value `%s' for `parm' is non-numeric", v + 5);
+                    nasm_nonfatal("value `%s' for `parm' is non-numeric", v + 5);
                     return DIRR_ERROR;
                 }
             } else {
                 bool err = false;
                 ordinal = readnum(v, &err);
                 if (err) {
-                    nasm_error(ERR_NONFATAL,
-                          "unrecognised export qualifier `%s'", v);
+                    nasm_nonfatal("unrecognised export qualifier `%s'", v);
                     return DIRR_ERROR;
                 }
                 flags |= EXPDEF_FLAG_ORDINAL;
@@ -1913,7 +1890,7 @@ static int32_t obj_segbase(int32_t segment)
             e = eb->exts[i];
 	    if (!e) {
                 /* Not available yet, probably a forward reference */
-		nasm_assert(pass0 < 2); /* Convergence failure */
+		nasm_assert(!pass_final());
 		return NO_SEG;
 	    }
 
@@ -1977,7 +1954,7 @@ static void obj_write_file(void)
     struct ExpDef *export;
     int lname_idx;
     ObjRecord *orp;
-    const StrList *depfile;
+    const struct strlist_entry *depfile;
     const bool debuginfo = (dfmt == &borland_debug_form);
 
     /*
@@ -1993,14 +1970,14 @@ static void obj_write_file(void)
      */
     orp->type = COMENT;
     obj_rword(orp, dTRANSL);
-    obj_name(orp, nasm_comment);
+    obj_name(orp, nasm_comment());
     obj_emit2(orp);
 
     /*
      * Output file dependency information
      */
-    if (!obj_nodepend) {
-        list_for_each(depfile, depend_list) {
+    if (!obj_nodepend && depend_list) {
+        strlist_for_each(depfile, depend_list) {
             uint32_t ts;
 
             ts = obj_file_timestamp(depfile->str);
@@ -2104,8 +2081,8 @@ static void obj_write_file(void)
             /* acbp |= 0x00 */ ;
         else if (seg->align >= 4096) {
             if (seg->align > 4096)
-                nasm_error(ERR_NONFATAL, "segment `%s' requires more alignment"
-                      " than OBJ format supports", seg->name);
+                nasm_nonfatal("segment `%s' requires more alignment"
+                              " than OBJ format supports", seg->name);
             acbp |= 0xC0;       /* PharLap extension */
         } else if (seg->align >= 256) {
             acbp |= 0x80;
@@ -2139,8 +2116,8 @@ static void obj_write_file(void)
 
         if (grp->nindices != grp->nentries) {
             for (i = grp->nindices; i < grp->nentries; i++) {
-                nasm_error(ERR_NONFATAL, "group `%s' contains undefined segment"
-                      " `%s'", grp->name, grp->segs[i].name);
+                nasm_nonfatal("group `%s' contains undefined segment"
+                              " `%s'", grp->name, grp->segs[i].name);
                 nasm_free(grp->segs[i].name);
                 grp->segs[i].name = NULL;
             }
@@ -2362,7 +2339,7 @@ static void obj_write_file(void)
             }
         }
         if (!seg)
-            nasm_error(ERR_NONFATAL, "entry point is not in this module");
+            nasm_nonfatal("entry point is not in this module");
     }
 
     /*
@@ -2527,19 +2504,19 @@ static void dbgbi_linnum(const char *lnfname, int32_t lineno, int32_t segto)
      * segment.
      */
     if (!any_segs) {
-        int tempint;            /* ignored */
-        if (segto != obj_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_panic(0, "strange segment conditions in OBJ driver");
+        int tempint = 0;
+        if (segto != obj_segment("__NASMDEFSEG", &tempint))
+            nasm_panic("strange segment conditions in OBJ driver");
     }
 
     /*
-     * Find the segment we are targetting.
+     * Find the segment we are targeting.
      */
     for (seg = seghead; seg; seg = seg->next)
         if (seg->index == segto)
             break;
     if (!seg)
-        nasm_panic(0, "lineno directed to nonexistent segment?");
+        nasm_panic("lineno directed to nonexistent segment?");
 
 /*    for (fn = fnhead; fn; fn = fnhead->next) */
     for (fn = fnhead; fn; fn = fn->next)        /* fbk - Austin Lunnen - John Fine */
@@ -2679,6 +2656,9 @@ static const struct dfmt borland_debug_form = {
     dbgbi_init,
     dbgbi_linnum,
     dbgbi_deflabel,
+    NULL,                       /* .debug_smacros */
+    NULL,                       /* .debug_include */
+    NULL,                       /* .debug_mmacros */
     null_debug_directive,
     dbgbi_typevalue,
     dbgbi_output,
@@ -2697,7 +2677,7 @@ static const struct pragma_facility obj_pragma_list[] = {
 };
 
 const struct ofmt of_obj = {
-    "MS-DOS 16-bit/32-bit OMF object files",
+    "Intel/Microsoft OMF (MS-DOS, OS/2, Win16)",
     "obj",
     ".obj",
     0,
