@@ -63,7 +63,7 @@ for cmd in ['new']:
                      required = False)
     spp.add_argument('--error',
                      dest = 'error',
-                     help = 'Set to "y" if test is supposed to fail',
+                     help = '"y" if test is supposed to fail or "i" to ignore',
                      required = False)
     spp.add_argument('--output',
                      dest = 'output', default = 'y',
@@ -117,7 +117,7 @@ if args.cmd == None:
 
 def read_stdfile(path):
     with open(path, "rb") as f:
-        data = f.read().decode("utf-8").strip("\n")
+        data = f.read().decode("utf-8")
         f.close()
         return data
 
@@ -182,7 +182,7 @@ def read_json(path):
     try:
         with open(path, "rb") as f:
             try:
-                desc = json.loads(f.read().decode("utf-8").strip("\n"))
+                desc = json.loads(f.read().decode("utf-8"))
             except:
                 desc = None
             finally:
@@ -297,8 +297,8 @@ def show_diff(test, patha, pathb):
     pb = run_hexdump(pathb)
     if pa == None or pb == None:
         return test_fail(test, "Can't create dumps")
-    sa = pa.stdout.read().decode("utf-8").strip("\n")
-    sb = pb.stdout.read().decode("utf-8").strip("\n")
+    sa = pa.stdout.read().decode("utf-8")
+    sb = pb.stdout.read().decode("utf-8")
     print("\t--- hexdump %s" % (patha))
     for i in sa.split("\n"):
         print("\t%s" % i)
@@ -361,8 +361,11 @@ def exec_nasm(desc):
         test_fail(desc['_test-name'], "Unable to execute test")
         return None
 
-    stderr = pnasm.stderr.read(4194304).decode("utf-8").strip("\n")
-    stdout = pnasm.stdout.read(4194304).decode("utf-8").strip("\n")
+    #
+    # FIXME: For now 4M buffer is enough but
+    # better provide reading in a cycle.
+    stderr = pnasm.stderr.read(4194304).decode("utf-8")
+    stdout = pnasm.stdout.read(4194304).decode("utf-8")
 
     pnasm.stdout.close()
     pnasm.stderr.close()
@@ -500,7 +503,9 @@ if args.cmd == 'new':
         if args.ref:
             acc.append("\t\t\"ref\": \"{}\"".format(args.ref))
         if args.error == 'y':
-            acc.append("\t\t\"error\": \"true\"")
+            acc.append("\t\t\"error\": \"expected\"")
+        elif args.error == 'i':
+            acc.append("\t\t\"error\": \"over\"")
         f.write(",\n".join(acc).encode("utf-8"))
         if args.output or args.stdout or args.stderr:
             acc = []
