@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 ## --------------------------------------------------------------------------
 ##
-##   Copyright 1996-2016 The NASM Authors - All Rights Reserved
+##   Copyright 1996-2020 The NASM Authors - All Rights Reserved
 ##   See the file AUTHORS included with the NASM distribution for
 ##   the specific copyright holders.
 ##
@@ -59,10 +59,118 @@ for ($c = 0; $c < $vex_classes; $c++) {
         }
     }
 }
-@disasm_prefixes = (@vexlist, @disasm_prefixes);
+@disasm_prefixes = (@vexlist, @disasm_prefixes, '');
+%disasm_prefixes = map { $_ => 1 } @disasm_prefixes;
 
 @bytecode_count = (0) x 256;
 
+# Push to an array reference, creating the array if needed
+sub xpush($@) {
+    my $ref = shift @_;
+
+    $$ref = [] unless (defined($$ref));
+    return push(@$$ref, @_);
+}
+
+# Generate relaxed form patterns if applicable
+sub relaxed_forms(@) {
+    my @field_list = @_;
+
+    foreach my $fields (@_) {
+	next unless ($fields->[1] =~ /\*/);
+
+	# This instruction has relaxed form(s)
+	if ($fields->[2] !~ /^\[/) {
+	    warn "$fname:$line: has an * operand but uses raw bytecodes\n";
+	    next;
+	}
+
+	my $opmask = 0;
+	my @ops = split(/,/, $fields->[1]);
+	for (my $oi = 0; $oi < scalar @ops; $oi++) {
+	    if ($ops[$oi] =~ /\*$/) {
+		if ($oi == 0) {
+		    warn "$fname:$line: has a first operand with a *\n";
+		    next;
+		}
+		$opmask |= 1 << $oi;
+	    }
+	}
+
+	for (my $oi = 1; $oi < (1 << scalar @ops); $oi++) {
+	    if (($oi & ~$opmask) == 0) {
+		my @xops = ();
+		my $omask = ~$oi;
+		for ($oj = 0; $oj < scalar(@ops); $oj++) {
+		    if ($omask & 1) {
+			push(@xops, $ops[$oj]);
+		    }
+		    $omask >>= 1;
+		}
+		my @ff = @$fields;
+		$ff[1] = join(',', @xops);
+		$ff[4] = $oi;
+		push(@field_list, [@ff]);
+	    }
+	}
+    }
+
+    return @field_list;
+}
+
+# Condition codes used by the disassembler
+my %condd = ( 'o'   =>  0, 'no'  =>  1, 'c'   =>  2,  'nc'  =>  3,
+	      'z'   =>  4, 'nz'  =>  5, 'na'  =>  6,  'a'   =>  7,
+	      's'   =>  8, 'ns'  =>  9, 'pe'  => 10,  'po'  => 11,
+	      'l'   => 12, 'nl'  => 13, 'ng'  => 14,  'g'   => 15 );
+
+# All condition code aliases
+my %conds = ( %condd,
+	      'ae'  =>  3, 'b'   =>  2, 'be'  =>  6,  'e'   =>  4,
+	      'ge'  => 13, 'le'  => 14, 'nae' =>  2,  'nb'  =>  3,
+	      'nbe' =>  7, 'ne'  =>  5, 'nge' => 12,  'nle' => 15,
+	      'np'  => 11, 'p'   => 10 );
+
+my @conds = sort keys(%conds);
+
+# Generate conditional form patterns if applicable
+sub conditional_forms(@) {
+    my @field_list = ();
+
+    foreach my $fields (@_) {
+	# This is a case sensitive match!
+	if ($fields->[0] !~ /cc/) {
+	    # Not a conditional instruction pattern
+	    push(@field_list, $fields);
+	    next;
+	}
+
+	if ($fields->[2] !~ /^\[/) {
+	    warn "$fname:$line: conditional instruction using raw bytecodes\n";
+	    next;
+	}
+
+	foreach my $cc (@conds) {
+	    my @ff = @$fields;
+
+	    $ff[0] =~ s/cc/\U$cc/;
+
+	    unless ($ff[2] =~ /^(\[.*?)\b([0-9a-f]{2})\+c\b(.*\])$/) {
+		warn "$fname:$line: invalid conditional encoding";
+		next;
+	    }
+	    $ff[2] = $1.sprintf('%02x', hex($2)^$conds{$cc}).$3;
+
+	    unless (defined($condd{$cc}) || $ff[3] =~ /\bND\b/) {
+		$ff[3] .= ',ND';
+	    }
+
+	    push(@field_list, [@ff]);
+	}
+    }
+    return @field_list;
+}
+
 print STDERR "Reading insns.dat...\n";
 
 @args   = ();
@@ -86,9 +194,13 @@ open(F, '<', $fname) || die "unable to open $fname";
 
 %dinstables = ();
 @bytecode_list = ();
+%aname = ();
 
 $line = 0;
 $insns = 0;
+$n_opcodes = 0;
+my @allpatterns = ();
+
 while (<F>) {
     $line++;
     chomp;
@@ -98,67 +210,24 @@ while (<F>) {
         warn "line $line does not contain four fields\n";
         next;
     }
-    @fields = ($1, $2, $3, $4);
-    @field_list = ([@fields, 0]);
+    my @field_list = ([$1, $2, $3, $4, 0]);
+    @field_list = relaxed_forms(@field_list);
+    @field_list = conditional_forms(@field_list);
 
-    if ($fields[1] =~ /\*/) {
-        # This instruction has relaxed form(s)
-        if ($fields[2] !~ /^\[/) {
-            warn "line $line has an * operand but uses raw bytecodes\n";
-            next;
-        }
-
-        $opmask = 0;
-        @ops = split(/,/, $fields[1]);
-        for ($oi = 0; $oi < scalar @ops; $oi++) {
-            if ($ops[$oi] =~ /\*$/) {
-                if ($oi == 0) {
-                    warn "line $line has a first operand with a *\n";
-                    next;
-                }
-                $opmask |= 1 << $oi;
-            }
-        }
-
-        for ($oi = 1; $oi < (1 << scalar @ops); $oi++) {
-            if (($oi & ~$opmask) == 0) {
-                my @xops = ();
-                my $omask = ~$oi;
-                for ($oj = 0; $oj < scalar(@ops); $oj++) {
-                    if ($omask & 1) {
-                        push(@xops, $ops[$oj]);
-                    }
-                    $omask >>= 1;
-                }
-                push(@field_list, [$fields[0], join(',', @xops),
-                     $fields[2], $fields[3], $oi]);
-            }
-        }
-    }
-
-    foreach $fptr (@field_list) {
-        @fields = @$fptr;
-        ($formatted, $nd) = format_insn(@fields);
+    foreach my $fields (@field_list) {
+        ($formatted, $nd) = format_insn(@$fields);
         if ($formatted) {
             $insns++;
-            $aname = "aa_$fields[0]";
-            push @$aname, $formatted;
-        }
-        if ( $fields[0] =~ /cc$/ ) {
-            # Conditional instruction
-            $k_opcodes_cc{$fields[0]}++;
-        } else {
-            # Unconditional instruction
-            $k_opcodes{$fields[0]}++;
+	    xpush(\$aname{$fields->[0]}, $formatted);
         }
+	if (!defined($k_opcodes{$fields->[0]})) {
+	    $k_opcodes{$fields->[0]} = $n_opcodes++;
+	}
         if ($formatted && !$nd) {
             push @big, $formatted;
-            my @sseq = startseq($fields[2], $fields[4]);
-            foreach $i (@sseq) {
-                if (!defined($dinstables{$i})) {
-                    $dinstables{$i} = [];
-                }
-                push(@{$dinstables{$i}}, $#big);
+            my @sseq = startseq($fields->[2], $fields->[4]);
+            foreach my $i (@sseq) {
+                xpush(\$dinstables{$i}, $#big);
             }
         }
     }
@@ -189,8 +258,7 @@ foreach $bl (@bytecode_list) {
 }
 undef @bytecode_list;
 
-@opcodes    = sort keys(%k_opcodes);
-@opcodes_cc = sort keys(%k_opcodes_cc);
+@opcodes = sort { $k_opcodes{$a} <=> $k_opcodes{$b} } keys(%k_opcodes);
 
 if ( $output eq 'b') {
     print STDERR "Writing $oname...\n";
@@ -244,16 +312,15 @@ if ( $output eq 'a' ) {
     print A "#include \"nasm.h\"\n";
     print A "#include \"insns.h\"\n\n";
 
-    foreach $i (@opcodes, @opcodes_cc) {
+    foreach $i (@opcodes) {
         print A "static const struct itemplate instrux_${i}[] = {\n";
-        $aname = "aa_$i";
-        foreach $j (@$aname) {
+        foreach $j (@{$aname{$i}}) {
             print A "    ", codesubst($j), "\n";
         }
         print A "    ITEMPLATE_END\n};\n\n";
     }
     print A "const struct itemplate * const nasm_instructions[] = {\n";
-    foreach $i (@opcodes, @opcodes_cc) {
+    foreach $i (@opcodes) {
         print A "    instrux_${i},\n";
     }
     print A "};\n";
@@ -281,7 +348,7 @@ if ( $output eq 'd' ) {
 
     foreach $h (sort(keys(%dinstables))) {
         next if ($h eq ''); # Skip pseudo-instructions
-            print D "\nstatic const struct itemplate * const itable_${h}[] = {\n";
+	print D "\nstatic const struct itemplate * const itable_${h}[] = {\n";
         foreach $j (@{$dinstables{$h}}) {
             print D "    instrux + $j,\n";
         }
@@ -289,7 +356,7 @@ if ( $output eq 'd' ) {
     }
 
     @prefix_list = ();
-    foreach $h (@disasm_prefixes, '') {
+    foreach $h (@disasm_prefixes) {
         for ($c = 0; $c < 256; $c++) {
             $nn = sprintf("%s%02X", $h, $c);
             if ($is_prefix{$nn} || defined($dinstables{$nn})) {
@@ -310,10 +377,15 @@ if ( $output eq 'd' ) {
         for ($c = 0; $c < 256; $c++) {
             $nn = sprintf("%s%02X", $h, $c);
             if ($is_prefix{$nn}) {
-                die "$fname: ambiguous decoding of $nn\n"
-                    if (defined($dinstables{$nn}));
+		if ($dinstables{$nn}) {
+		    print STDERR "$fname: ambiguous decoding, prefix $nn aliases:\n";
+		    foreach my $dc (@{$dinstables{$nn}}) {
+			print STDERR codesubst($big[$dc]), "\n";
+		    }
+		    exit 1;
+		}
                 printf D "    /* 0x%02x */ { itable_%s, -1 },\n", $c, $nn;
-            } elsif (defined($dinstables{$nn})) {
+            } elsif ($dinstables{$nn}) {
                 printf D "    /* 0x%02x */ { itable_%s, %u },\n", $c,
                        $nn, scalar(@{$dinstables{$nn}});
             } else {
@@ -357,10 +429,9 @@ if ( $output eq 'i' ) {
     print I "#define NASM_INSNSI_H 1\n\n";
     print I "enum opcode {\n";
     $maxlen = 0;
-    foreach $i (@opcodes, @opcodes_cc) {
+    foreach $i (@opcodes) {
         print I "\tI_${i},\n";
         $len = length($i);
-        $len++ if ( $i =~ /cc$/ ); # Condition codes can be 3 characters long
         $maxlen = $len if ( $len > $maxlen );
     }
     print I "\tI_none = -1\n";
@@ -368,7 +439,6 @@ if ( $output eq 'i' ) {
     print I "#define MAX_INSLEN ", $maxlen, "\n";
     print I "#define NASM_VEX_CLASSES ", $vex_classes, "\n";
     print I "#define NO_DECORATOR\t{", join(',',(0) x $MAX_OPERANDS), "}\n";
-    print I "#define FIRST_COND_OPCODE I_", $opcodes_cc[0], "\n\n";
     print I "#endif /* NASM_INSNSI_H */\n";
 
     close I;
@@ -384,14 +454,9 @@ if ( $output eq 'n' ) {
     print N "#include \"tables.h\"\n\n";
 
     print N "const char * const nasm_insn_names[] = {";
-    $first = 1;
-    foreach $i (@opcodes, @opcodes_cc) {
-        print N "," if ( !$first );
-        $first = 0;
-        $ilower = $i;
-        $ilower =~ s/cc$//;             # Remove conditional cc suffix
-        $ilower =~ tr/A-Z/a-z/;         # Change to lower case (Perl 4 compatible)
-        print N "\n\t\"${ilower}\"";
+    foreach $i (@opcodes) {
+        print N "\n\t\"\L$i\"";
+	print N ',' if ($i < $#opcodes);
     }
     print N "\n};\n";
     close N;
@@ -436,9 +501,10 @@ sub count_bytecodes(@) {
 
 sub format_insn($$$$$) {
     my ($opcode, $operands, $codes, $flags, $relax) = @_;
-    my $num, $nd = 0, $rawflags, $flagsindex;
+    my $nd = 0;
+    my ($num, $flagsindex);
     my @bytecode;
-    my $op, @ops, $opp, @opx, @oppx, @decos, @opevex;
+    my ($op, @ops, @opsize, $opp, @opx, @oppx, @decos, @opevex);
 
     return (undef, undef) if $operands eq "ignore";
 
@@ -446,19 +512,22 @@ sub format_insn($$$$$) {
     $operands =~ s/\*//g;
     $operands =~ s/:/|colon,/g;
     @ops = ();
+    @opsize = ();
     @decos = ();
     if ($operands ne 'void') {
         foreach $op (split(/,/, $operands)) {
+	    my $opsz = 0;
             @opx = ();
             @opevex = ();
             foreach $opp (split(/\|/, $op)) {
                 @oppx = ();
-                if ($opp =~ s/^(b(32|64)|mask|z|er|sae)$//) {
+                if ($opp =~ s/^(b(16|32|64)|mask|z|er|sae)$//) {
                     push(@opevex, $1);
                 }
 
                 if ($opp =~ s/(?<!\d)(8|16|32|64|80|128|256|512)$//) {
                     push(@oppx, "bits$1");
+		    $opsz = $1 + 0;
                 }
                 $opp =~ s/^mem$/memory/;
                 $opp =~ s/^memory_offs$/mem_offs/;
@@ -475,6 +544,7 @@ sub format_insn($$$$$) {
             }
             $op = join('|', @opx);
             push(@ops, $op);
+	    push(@opsize, $opsz);
             push(@decos, (@opevex ? join('|', @opevex) : '0'));
         }
     }
@@ -482,6 +552,7 @@ sub format_insn($$$$$) {
     $num = scalar(@ops);
     while (scalar(@ops) < $MAX_OPERANDS) {
         push(@ops, '0');
+	push(@opsize, 0);
         push(@decos, '0');
     }
     $operands = join(',', @ops);
@@ -493,19 +564,71 @@ sub format_insn($$$$$) {
     }
     $decorators =~ tr/a-z/A-Z/;
 
-    # format the flags
-    $nd = 1 if $flags =~ /(^|\,)ND($|\,)/;
-    $flags =~ s/(^|\,)ND($|\,)/\1/g;
-    $flags =~ s/(^|\,)X64($|\,)/\1LONG,X86_64\2/g;
-    if ($codes =~ /evex\./) {
-	$flags .= ",EVEX";
-    } elsif ($codes =~ /(vex|xop)\./) {
-	$flags .= ",VEX";
-    }
-    $rawflags = $flags;
-    $flagsindex = insns_flag_index(split(',',$flags));
+    # Remember if we have an ARx flag
+    my $arx = undef;
 
-    die "Error in flags $rawflags" if not defined($flagsindex);
+    # expand and uniqify the flags
+    my %flags;
+    foreach my $flag (split(',', $flags)) {
+	next if ($flag eq '');
+
+	if ($flag eq 'ND') {
+	    $nd = 1;
+	} else {
+	    $flags{$flag}++;
+	}
+
+	if ($flag eq 'NEVER' || $flag eq 'NOP') {
+	    # These flags imply OBSOLETE
+	    $flags{'OBSOLETE'}++;
+	}
+
+	if ($flag =~ /^AR([0-9]+)$/) {
+	    $arx = $1+0;
+	}
+    }
+
+    if ($codes =~ /evex\./) {
+	$flags{'EVEX'}++;
+    } elsif ($codes =~ /(vex|xop)\./) {
+	$flags{'VEX'}++;
+    }
+
+    # Look for SM flags clearly inconsistent with operand bitsizes
+    if ($flags{'SM'} || $flags{'SM2'}) {
+	my $ssize = 0;
+	my $e = $flags{'SM2'} ? 2 : $MAX_OPERANDS;
+	for (my $i = 0; $i < $e; $i++) {
+	    next if (!$opsize[$i]);
+	    if (!$ssize) {
+		$ssize = $opsize[$i];
+	    } elsif ($opsize[$i] != $ssize) {
+		die "$fname:$line: inconsistent SM flag for argument $i\n";
+	    }
+	}
+    }
+
+    # Look for Sx flags that can never match operand bitsizes. If the
+    # intent is to never match (require explicit sizes), use the SX flag.
+    # This doesn't apply to registers that pre-define specific sizes;
+    # this should really be derived from include/opflags.h...
+    my %sflags = ( 'SB' => 8, 'SW' => 16, 'SD' => 32, 'SQ' => 64,
+		   'SO' => 128, 'SY' => 256, 'SZ' => 512 );
+    my $s = defined($arx) ? $arx : 0;
+    my $e = defined($arx) ? $arx : $MAX_OPERANDS - 1;
+
+    foreach my $sf (keys(%sflags)) {
+	next if (!$flags{$sf});
+	for (my $i = $s; $i <= $e; $i++) {
+	    if ($opsize[$i] && $ops[$i] !~ /\breg_(gpr|[cdts]reg)\b/) {
+		die "$fname:$line: inconsistent $sf flag for argument $i ($ops[$i])\n"
+		    if ($opsize[$i] != $sflags{$sf});
+	    }
+	}
+    }
+
+    $flagsindex = insns_flag_index(keys %flags);
+    die "$fname:$line: error in flags $flags\n" unless (defined($flagsindex));
 
     @bytecode = (decodify($codes, $relax), 0);
     push(@bytecode_list, [@bytecode]);
@@ -526,7 +649,7 @@ sub codesubst($) {
     while ($s =~ /\@\@CODES-([0-9A-F]+)\@\@/) {
         my $pos = $bytecode_pos{$1};
         if (!defined($pos)) {
-            die "$fname: no position assigned to byte code $1\n";
+            die "$fname:$line: no position assigned to byte code $1\n";
         }
         $s = $` . "nasm_bytecodes+${pos}" . "$'";
     }
@@ -572,7 +695,7 @@ sub decodify($$) {
                 $c = $2;
                 next;
             } else {
-                die "$fname: unknown code format in \"$codestr\"\n";
+                die "$fname:$line: unknown code format in \"$codestr\"\n";
             }
         }
     }
@@ -595,7 +718,6 @@ sub hexstr(@) {
 # instruction. We need only consider the codes:
 # \[1234]      mean literal bytes, of course
 # \1[0123]     mean byte plus register value
-# \330         means byte plus condition code
 # \0 or \340   mean give up and return empty set
 # \34[4567]    mean PUSH/POP of segment registers: special case
 # \17[234]     skip is4 control byte
@@ -603,20 +725,20 @@ sub hexstr(@) {
 # \24x \250    skip EVEX control bytes
 sub startseq($$) {
     my ($codestr, $relax) = @_;
-    my $word, @range;
+    my $word;
     my @codes = ();
     my $c = $codestr;
-    my $c0, $c1, $i;
+    my($c0, $c1, $i);
     my $prefix = '';
 
     @codes = decodify($codestr, $relax);
 
-    while ($c0 = shift(@codes)) {
+    while (defined($c0 = shift(@codes))) {
         $c1 = $codes[0];
         if ($c0 >= 01 && $c0 <= 04) {
             # Fixed byte string
             my $fbs = $prefix;
-            while (1) {
+            while (defined($c0)) {
                 if ($c0 >= 01 && $c0 <= 04) {
                     while ($c0--) {
                         $fbs .= sprintf("%02X", shift(@codes));
@@ -628,15 +750,16 @@ sub startseq($$) {
             }
 
             foreach $pfx (@disasm_prefixes) {
-                if (substr($fbs, 0, length($pfx)) eq $pfx) {
+		my $len = length($pfx);
+                if (substr($fbs, 0, $len) eq $pfx) {
                     $prefix = $pfx;
-                    $fbs = substr($fbs, length($pfx));
+                    $fbs = substr($fbs, $len, 2);
                     last;
                 }
             }
 
             if ($fbs ne '') {
-                return ($prefix.substr($fbs,0,2));
+                return ($prefix.$fbs);
             }
 
             unshift(@codes, $c0);
@@ -644,13 +767,11 @@ sub startseq($$) {
             return addprefix($prefix, $c1..($c1+7));
         } elsif (($c0 & ~013) == 0144) {
             return addprefix($prefix, $c1, $c1|2);
-        } elsif ($c0 == 0330) {
-            return addprefix($prefix, $c1..($c1+15));
         } elsif ($c0 == 0 || $c0 == 0340) {
             return $prefix;
         } elsif (($c0 & ~3) == 0260 || $c0 == 0270 ||
                  ($c0 & ~3) == 0240 || $c0 == 0250) {
-            my $c,$m,$wlp;
+            my($c,$m,$wlp);
             $m   = shift(@codes);
             $wlp = shift(@codes);
             $c = ($m >> 6);
@@ -666,7 +787,7 @@ sub startseq($$) {
             # and "ignorable" codes here
         }
     }
-    return $prefix;
+    return ();
 }
 
 # EVEX tuple types offset is 0300. e.g. 0301 is for full vector(fv).
@@ -695,7 +816,7 @@ sub tupletype($) {
     if (defined $tuple_codes{$tuplestr}) {
         return 0300 + $tuple_codes{$tuplestr};
     } else {
-        die "Undefined tuple type : $tuplestr\n";
+        die "$fname:$line: undefined tuple type : $tuplestr\n";
     }
 }
 
@@ -728,7 +849,7 @@ sub byte_code_compile($$) {
     my $litix = undef;
     my %oppos = ();
     my $i;
-    my $op, $oq;
+    my ($op, $oq);
     my $opex;
 
     my %imm_codes = (
@@ -799,13 +920,13 @@ sub byte_code_compile($$) {
     );
 
     unless ($str =~ /^(([^\s:]*)\:*([^\s:]*)\:|)\s*(.*\S)\s*$/) {
-        die "$fname: $line: cannot parse: [$str]\n";
+        die "$fname:$line: cannot parse: [$str]\n";
     }
-    $opr = "\L$2";
-    $tuple = "\L$3";    # Tuple type for AVX512
-    $opc = "\L$4";
+    $opr = lc($2);
+    $tuple = lc($3);    # Tuple type for AVX512
+    $opc = lc($4);
 
-    my $op = 0;
+    $op = 0;
     for ($i = 0; $i < length($opr); $i++) {
         my $c = substr($opr,$i,1);
         if ($c eq '+') {
@@ -849,7 +970,7 @@ sub byte_code_compile($$) {
             $prefix_ok = 0;
         } elsif ($op eq '/r') {
             if (!defined($oppos{'r'}) || !defined($oppos{'m'})) {
-                die "$fname: $line: $op requires r and m operands\n";
+                die "$fname:$line: $op requires r and m operands\n";
             }
             $opex = (($oppos{'m'} & 4) ? 06 : 0) |
                 (($oppos{'r'} & 4) ? 05 : 0);
@@ -860,11 +981,19 @@ sub byte_code_compile($$) {
             $prefix_ok = 0;
         } elsif ($op =~ m:^/([0-7])$:) {
             if (!defined($oppos{'m'})) {
-                die "$fname: $line: $op requires m operand\n";
+                die "$fname:$line: $op requires an m operand\n";
             }
             push(@codes, 06) if ($oppos{'m'} & 4);
             push(@codes, 0200 + (($oppos{'m'} & 3) << 3) + $1);
             $prefix_ok = 0;
+	} elsif ($op =~ m:^/([0-3]?)r([0-7])$:) {
+	    if (!defined($oppos{'r'})) {
+                die "$fname:$line: $op requires an r operand\n";
+	    }
+	    push(@codes, 05) if ($oppos{'r'} & 4);
+	    push(@codes, 0171);
+	    push(@codes, (($1+0) << 6) + (($oppos{'r'} & 3) << 3) + $2);
+	    $prefix_ok = 0;
         } elsif ($op =~ /^(vex|xop)(|\..*)$/) {
             my $vexname = $1;
             my $c = $vexmap{$vexname};
@@ -887,7 +1016,7 @@ sub byte_code_compile($$) {
                         $w = 2;
                     } elsif ($oq eq 'ww') {
                         $w = 3;
-                    } elsif ($oq eq 'p0') {
+                    } elsif ($oq eq 'np' || $oq eq 'p0') {
                         $p = 0;
                     } elsif ($oq eq '66' || $oq eq 'p1') {
                         $p = 1;
@@ -901,26 +1030,23 @@ sub byte_code_compile($$) {
                         $m = 2;
                     } elsif ($oq eq '0f3a') {
                         $m = 3;
-                    } elsif ($oq =~ /^m([0-9]+)$/) {
-                        $m = $1+0;
+                    } elsif ($oq =~ /^(m|map)([0-9]+)$/) {
+                        $m = $2+0;
                     } elsif ($oq eq 'nds' || $oq eq 'ndd' || $oq eq 'dds') {
                         if (!defined($oppos{'v'})) {
-                            die "$fname: $line: $vexname.$oq without 'v' operand\n";
+                            die "$fname:$line: $vexname.$oq without 'v' operand\n";
                         }
                         $has_nds = 1;
                     } else {
-                        die "$fname: $line: undefined \U$vexname\E subcode: $oq\n";
+                        die "$fname:$line: undefined \U$vexname\E subcode: $oq\n";
                     }
                 }
             if (!defined($m) || !defined($w) || !defined($l) || !defined($p)) {
-                die "$fname: $line: missing fields in \U$vexname\E specification\n";
-            }
-            if (defined($oppos{'v'}) && !$has_nds) {
-                die "$fname: $line: 'v' operand without ${vexname}.nds or ${vexname}.ndd\n";
+                die "$fname:$line: missing fields in \U$vexname\E specification\n";
             }
 	    my $minmap = ($c == 1) ? 8 : 0; # 0-31 for VEX, 8-31 for XOP
 	    if ($m < $minmap || $m > 31) {
-		die "$fname: $line: Only maps ${minmap}-31 are valid for \U${vexname}\n";
+		die "$fname:$line: Only maps ${minmap}-31 are valid for \U${vexname}\n";
 	    }
             push(@codes, defined($oppos{'v'}) ? 0260+($oppos{'v'} & 3) : 0270,
                  ($c << 6)+$m, ($w << 4)+($l << 2)+$p);
@@ -946,7 +1072,7 @@ sub byte_code_compile($$) {
                         $w = 2;
                     } elsif ($oq eq 'ww') {
                         $w = 3;
-                    } elsif ($oq eq 'p0') {
+                    } elsif ($oq eq 'np' || $oq eq 'p0') {
                         $p = 0;
                     } elsif ($oq eq '66' || $oq eq 'p1') {
                         $p = 1;
@@ -960,25 +1086,26 @@ sub byte_code_compile($$) {
                         $m = 2;
                     } elsif ($oq eq '0f3a') {
                         $m = 3;
+                    } elsif ($oq eq 'map5') {
+                        $m = 5;
+                    } elsif ($oq eq 'map6') {
+                        $m = 6;
                     } elsif ($oq =~ /^m([0-9]+)$/) {
                         $m = $1+0;
                     } elsif ($oq eq 'nds' || $oq eq 'ndd' || $oq eq 'dds') {
                         if (!defined($oppos{'v'})) {
-                            die "$fname: $line: evex.$oq without 'v' operand\n";
+                            die "$fname:$line: evex.$oq without 'v' operand\n";
                         }
                         $has_nds = 1;
                     } else {
-                        die "$fname: $line: undefined EVEX subcode: $oq\n";
+                        die "$fname:$line: undefined EVEX subcode: $oq\n";
                     }
                 }
             if (!defined($m) || !defined($w) || !defined($l) || !defined($p)) {
-                die "$fname: $line: missing fields in EVEX specification\n";
-            }
-            if (defined($oppos{'v'}) && !$has_nds) {
-                die "$fname: $line: 'v' operand without evex.nds or evex.ndd\n";
+                die "$fname:$line: missing fields in EVEX specification\n";
             }
 	    if ($m > 15) {
-		die "$fname: $line: Only maps 0-15 are valid for EVEX\n";
+		die "$fname:$line: Only maps 0-15 are valid for EVEX\n";
 	    }
             push(@codes, defined($oppos{'v'}) ? 0240+($oppos{'v'} & 3) : 0250,
                  ($c << 6)+$m, ($w << 4)+($l << 2)+$p, $tup);
@@ -986,23 +1113,23 @@ sub byte_code_compile($$) {
         } elsif (defined $imm_codes{$op}) {
             if ($op eq 'seg') {
                 if ($last_imm lt 'i') {
-                    die "$fname: $line: seg without an immediate operand\n";
+                    die "$fname:$line: seg without an immediate operand\n";
                 }
             } else {
                 $last_imm++;
                 if ($last_imm gt 'j') {
-                    die "$fname: $line: too many immediate operands\n";
+                    die "$fname:$line: too many immediate operands\n";
                 }
             }
             if (!defined($oppos{$last_imm})) {
-                die "$fname: $line: $op without '$last_imm' operand\n";
+                die "$fname:$line: $op without '$last_imm' operand\n";
             }
             push(@codes, 05) if ($oppos{$last_imm} & 4);
             push(@codes, $imm_codes{$op} + ($oppos{$last_imm} & 3));
             $prefix_ok = 0;
         } elsif ($op eq '/is4') {
             if (!defined($oppos{'s'})) {
-                die "$fname: $line: $op without 's' operand\n";
+                die "$fname:$line: $op without 's' operand\n";
             }
             if (defined($oppos{'i'})) {
                 push(@codes, 0172, ($oppos{'s'} << 3)+$oppos{'i'});
@@ -1014,19 +1141,16 @@ sub byte_code_compile($$) {
         } elsif ($op =~ /^\/is4\=([0-9]+)$/) {
             my $imm = $1;
             if (!defined($oppos{'s'})) {
-                die "$fname: $line: $op without 's' operand\n";
+                die "$fname:$line: $op without 's' operand\n";
             }
             if ($imm < 0 || $imm > 15) {
-                die "$fname: $line: invalid imm4 value for $op: $imm\n";
+                die "$fname:$line: invalid imm4 value for $op: $imm\n";
             }
             push(@codes, 0173, ($oppos{'s'} << 4) + $imm);
             $prefix_ok = 0;
-        } elsif ($op =~ /^([0-9a-f]{2})\+c$/) {
-            push(@codes, 0330, hex $1);
-            $prefix_ok = 0;
         } elsif ($op =~ /^([0-9a-f]{2})\+r$/) {
             if (!defined($oppos{'r'})) {
-                die "$fname: $line: $op without 'r' operand\n";
+                die "$fname:$line: $op without 'r' operand\n";
             }
             push(@codes, 05) if ($oppos{'r'} & 4);
             push(@codes, 010 + ($oppos{'r'} & 3), hex $1);
@@ -1035,7 +1159,7 @@ sub byte_code_compile($$) {
             # Escape to enter literal bytecodes
             push(@codes, oct $1);
         } else {
-            die "$fname: $line: unknown operation: $op\n";
+            die "$fname:$line: unknown operation: $op\n";
         }
     }
 
